@@ -35,11 +35,6 @@ class MainWindow(object):
     timer = None
     #hack to make editing menu properties work
     allow_update = True
-    #drag-and-drop stuff
-    dnd_items = [('ALACARTE_ITEM_ROW', Gtk.TargetFlags.SAME_APP, 0), ('text/plain', 0, 1)]
-    dnd_menus = [('ALACARTE_MENU_ROW', Gtk.TargetFlags.SAME_APP, 0)]
-    dnd_both = [dnd_items[0],] + dnd_menus
-    drag_data = None
     edit_pool = []
 
     def __init__(self, datadir, version):
@@ -61,10 +56,6 @@ class MainWindow(object):
         self.tree.get_object('move_down_button').set_sensitive(False)
         self.tree.get_object('new_separator_button').set_sensitive(False)
         accelgroup = Gtk.AccelGroup()
-        keyval, modifier = Gtk.accelerator_parse('<Ctrl>Z')
-        accelgroup.connect(keyval, modifier, Gtk.AccelFlags.VISIBLE, self.on_mainwindow_undo)
-        keyval, modifier = Gtk.accelerator_parse('<Ctrl><Shift>Z')
-        accelgroup.connect(keyval, modifier, Gtk.AccelFlags.VISIBLE, self.on_mainwindow_redo)
         keyval, modifier = Gtk.accelerator_parse('F1')
         accelgroup.connect(keyval, modifier, Gtk.AccelFlags.VISIBLE, self.on_help_button_clicked)
         self.tree.get_object('mainwindow').add_accel_group(accelgroup)
@@ -170,8 +161,6 @@ class MainWindow(object):
         column.pack_start(cell, True)
         column.add_attribute(cell, 'markup', 1)
         menus.append_column(column)
-        menus.enable_model_drag_source(Gdk.ModifierType.BUTTON1_MASK, self.dnd_menus, Gdk.DragAction.COPY)
-        menus.enable_model_drag_dest(self.dnd_both, Gdk.DragAction.PRIVATE)
         menus.get_selection().set_mode(Gtk.SelectionMode.BROWSE)
 
     def setupItemTree(self):
@@ -195,8 +184,6 @@ class MainWindow(object):
         items.append_column(column)
         self.item_store = Gtk.ListStore(bool, GdkPixbuf.Pixbuf, str, object)
         items.set_model(self.item_store)
-        items.enable_model_drag_source(Gdk.ModifierType.BUTTON1_MASK, self.dnd_items, Gdk.DragAction.COPY)
-        items.enable_model_drag_dest(self.dnd_items, Gdk.DragAction.PRIVATE)
 
     def _cell_data_toggle_func(self, tree_column, renderer, model, treeiter, data=None):
         if isinstance(model[treeiter][3], GMenu.TreeSeparator):
@@ -206,9 +193,8 @@ class MainWindow(object):
 
     def loadMenus(self):
         self.menu_store.clear()
-        for menu in self.editor.getMenus():
-            iters = [None]*20
-            self.loadMenu(iters, menu)
+        self.loadMenu({ None: None })
+
         menu_tree = self.tree.get_object('menu_tree')
         menu_tree.set_model(self.menu_store)
         for menu in self.menu_store:
@@ -216,54 +202,46 @@ class MainWindow(object):
         menu_tree.get_selection().select_path((0,))
         self.on_menu_tree_cursor_changed(menu_tree)
 
-    def loadMenu(self, iters, parent, depth=0):
-        if depth == 0:
-            icon = util.getIcon(parent)
-            iters[depth] = self.menu_store.append(None, (icon, cgi.escape(parent.get_name()), parent))
-        depth += 1
+    def loadMenu(self, iters, parent=None):
         for menu, show in self.editor.getMenus(parent):
-            if show:
-                name = cgi.escape(menu.get_name())
-            else:
-                name = '<small><i>' + cgi.escape(menu.get_name()) + '</i></small>'
+            name = cgi.escape(menu.get_name())
+            if not show:
+                name = "<small><i>%s</i></small>" % (name,)
+
             icon = util.getIcon(menu)
-            iters[depth] = self.menu_store.append(iters[depth-1], (icon, name, menu))
-            self.loadMenu(iters, menu, depth)
-        depth -= 1
+            iters[menu] = self.menu_store.append(iters[parent], (icon, name, menu))
+            self.loadMenu(iters, menu)
 
     def loadItems(self, menu, menu_path):
         self.item_store.clear()
         for item, show in self.editor.getItems(menu):
-            menu_icon = None
-            if isinstance(item, GMenu.TreeSeparator):
-                name = '---'
-                icon = None
+            icon = util.getIcon(item)
+            if isinstance(item, GMenu.TreeDirectory):
+                name = item.get_name()
             elif isinstance(item, GMenu.TreeEntry):
-                app_info = item.get_app_info()
-                if show:
-                    name = cgi.escape(app_info.get_display_name())
-                else:
-                    name = '<small><i>' + cgi.escape(app_info.get_display_name()) + '</i></small>'
-                icon = util.getIcon(item)
+                name = item.get_app_info().get_display_name()
+            elif isinstance(item, GMenu.Separator):
+                name = '---'
             else:
-                if show:
-                    name = cgi.escape(item.get_name())
-                else:
-                    name = '<small><i>' + cgi.escape(item.get_name()) + '</i></small>'
-                icon = util.getIcon(item)
+                assert False, 'should not be reached'
+
+            name = cgi.escape(name)
+            if not show:
+                name = "<small><i>%s</i></small>" % (name,)
+
             self.item_store.append((show, icon, name, item))
 
     #this is a little timeout callback to insert new items after
     #gnome-desktop-item-edit has finished running
     def waitForNewItemProcess(self, process, parent_id, file_path):
-        if process.poll() != None:
+        if process.poll() is not None:
             if os.path.isfile(file_path):
                 self.editor.insertExternalItem(os.path.split(file_path)[1], parent_id)
             return False
         return True
 
     def waitForNewMenuProcess(self, process, parent_id, file_path):
-        if process.poll() != None:
+        if process.poll() is not None:
             if os.path.isfile(file_path):
                 self.editor.insertExternalMenu(os.path.split(file_path)[1], parent_id)
             return False
@@ -271,7 +249,7 @@ class MainWindow(object):
 
     #this callback keeps you from editing the same item twice
     def waitForEditProcess(self, process, file_path):
-        if process.poll() != None:
+        if process.poll() is not None:
             self.edit_pool.remove(file_path)
             return False
         return True
@@ -357,16 +335,17 @@ class MainWindow(object):
         if not os.path.isfile(file_path):
             data = open(item.get_desktop_file_path()).read()
             open(file_path, 'w').write(data)
-            self.editor._MenuEditor__addUndo([(file_type, os.path.split(file_path)[1]),])
-        else:
-            self.editor._MenuEditor__addUndo([item,])
+
         if file_path not in self.edit_pool:
             self.edit_pool.append(file_path)
             process = subprocess.Popen(['gnome-desktop-item-edit', file_path], env=os.environ)
             GObject.timeout_add(100, self.waitForEditProcess, process, file_path)
 
     def on_menu_tree_cursor_changed(self, treeview):
-        menus, iter = treeview.get_selection().get_selected()
+        selection = treeview.get_selection()
+        if selection is None:
+            return
+        menus, iter = selection.get_selected()
         if iter is None:
             return
         menu_path = menus.get_path(iter)
@@ -382,37 +361,6 @@ class MainWindow(object):
         self.tree.get_object('properties_button').set_sensitive(False)
         self.tree.get_object('delete_button').set_sensitive(False)
 
-    def on_menu_tree_drag_data_get(self, treeview, context, selection, target_id, etime):
-        menus, iter = treeview.get_selection().get_selected()
-        self.drag_data = menus[iter][2]
-
-    def on_menu_tree_drag_data_received(self, treeview, context, x, y, selection, info, etime):
-        menus = treeview.get_model()
-        drop_info = treeview.get_dest_row_at_pos(x, y)
-        types = (Gtk.TreeViewDropPosition.BEFORE, Gtk.TreeViewDropPosition.INTO_OR_BEFORE)
-        if drop_info:
-            path, position = drop_info
-            if position not in types:
-                context.finish(False, False, etime)
-                return False
-            if selection.target in ('ALACARTE_ITEM_ROW', 'ALACARTE_MENU_ROW'):
-                if self.drag_data == None:
-                    return False
-                item = self.drag_data
-                new_parent = menus[path][2]
-                treeview.get_selection().select_path(path)
-                if isinstance(item, GMenu.TreeEntry):
-                    self.editor.copyItem(item, new_parent)
-                elif isinstance(item, GMenu.TreeDirectory):
-                    if self.editor.moveMenu(item, new_parent) == False:
-                        self.loadUpdates()
-                elif isinstance(item, GMenu.TreeSeparator):
-                    self.editor.moveSeparator(item, new_parent)
-                else:
-                    context.finish(False, False, etime) 
-                context.finish(True, True, etime)
-        self.drag_data = None
-
     def on_item_tree_show_toggled(self, cell, path):
         item = self.item_store[path][3]
         if isinstance(item, GMenu.TreeSeparator):
@@ -424,35 +372,30 @@ class MainWindow(object):
         self.item_store[path][0] = not self.item_store[path][0]
 
     def on_item_tree_cursor_changed(self, treeview):
-        items, iter = treeview.get_selection().get_selected()
+        selection = treeview.get_selection()
+        if selection is None:
+            return
+        items, iter = selection.get_selected()
         if iter is None:
             return
+
         item = items[iter][3]
         self.tree.get_object('edit_delete').set_sensitive(True)
         self.tree.get_object('new_separator_button').set_sensitive(True)
         self.tree.get_object('delete_button').set_sensitive(True)
-        if self.editor.canRevert(item):
-            self.tree.get_object('edit_revert_to_original').set_sensitive(True)
-        else:
-            self.tree.get_object('edit_revert_to_original').set_sensitive(False)
-        if not isinstance(item, GMenu.TreeSeparator):
-            self.tree.get_object('edit_properties').set_sensitive(True)
-            self.tree.get_object('properties_button').set_sensitive(True)
-        else:
-            self.tree.get_object('edit_properties').set_sensitive(False)
-            self.tree.get_object('properties_button').set_sensitive(False)
 
-        # If first item...
-        if items.get_path(iter).get_indices()[0] == 0:
-            self.tree.get_object('move_up_button').set_sensitive(False)
-        else:
-            self.tree.get_object('move_up_button').set_sensitive(True)
+        can_revert = self.editor.canRevert(item)
+        self.tree.get_object('edit_revert_to_original').set_sensitive(can_revert)
 
-        # If last item...
-        if items.get_path(iter).get_indices()[0] == (len(items)-1):
-            self.tree.get_object('move_down_button').set_sensitive(False)
-        else:
-            self.tree.get_object('move_down_button').set_sensitive(True)
+        can_edit = not isinstance(item, GMenu.TreeSeparator)
+        self.tree.get_object('edit_properties').set_sensitive(can_edit)
+        self.tree.get_object('properties_button').set_sensitive(can_edit)
+
+        index = items.get_path(iter).get_indices()[0]
+        can_go_up = index > 0
+        can_go_down = index < len(items) - 1
+        self.tree.get_object('move_up_button').set_sensitive(can_go_up)
+        self.tree.get_object('move_down_button').set_sensitive(can_go_down)
 
     def on_item_tree_row_activated(self, treeview, path, column):
         self.on_edit_properties_activate(None)
@@ -466,7 +409,7 @@ class MainWindow(object):
             button = event.button
             event_time = event.time
             info = item_tree.get_path_at_pos(int(event.x), int(event.y))
-            if info != None:
+            if info is not None:
                 path, col, cellx, celly = info
                 item_tree.grab_focus()
                 item_tree.set_cursor(path, col, 0)
@@ -481,84 +424,6 @@ class MainWindow(object):
         #without this shift-f10 won't work
         return True
 
-    def on_item_tree_drag_data_get(self, treeview, context, selection, target_id, etime):
-        items, iter = treeview.get_selection().get_selected()
-        self.drag_data = items[iter][3]
-
-    def on_item_tree_drag_data_received(self, treeview, context, x, y, selection, info, etime):
-        items = treeview.get_model()
-        types_into = (Gtk.TreeViewDropPosition.INTO_OR_BEFORE, Gtk.TreeViewDropPosition.INTO_OR_AFTER)
-        types_before = (Gtk.TreeViewDropPosition.BEFORE, Gtk.TreeViewDropPosition.INTO_OR_BEFORE)
-        types_after = (Gtk.TreeViewDropPosition.AFTER, Gtk.TreeViewDropPosition.INTO_OR_AFTER)
-        if selection.target == 'ALACARTE_ITEM_ROW':
-            drop_info = treeview.get_dest_row_at_pos(x, y)
-            before = None
-            after = None
-            if self.drag_data == None:
-                return False
-            item = self.drag_data
-            # by default we assume, that the items stays in the same menu
-            destination = item.get_parent()
-            if drop_info:
-                path, position = drop_info
-                target = items[path][3]
-                # move the item to the directory, if the item was dropped into it
-                if isinstance(target, GMenu.TreeDirectory) and (position in types_into):
-                    # append the selected item to the choosen menu
-                    destination = target
-                elif position in types_before:
-                    before = target
-                elif position in types_after:
-                    after = target
-                else:
-                    # this does not happen
-                    pass
-            else:
-                path = (len(items) - 1,)
-                after = items[path][3]
-            if isinstance(item, GMenu.TreeEntry):
-                self.editor.moveItem(item, destination, before, after)
-            elif isinstance(item, GMenu.TreeDirectory):
-                if self.editor.moveMenu(item, destination, before, after) == False:
-                    self.loadUpdates()
-            elif isinstance(item, GMenu.TreeSeparator):
-                self.editor.moveSeparator(item, destination, before, after)
-            context.finish(True, True, etime)
-        elif selection.target == 'text/plain':
-            if selection.data == None:
-                return False
-            menus, iter = self.tree.get_object('menu_tree').get_selection().get_selected()
-            parent = menus[iter][2]
-            drop_info = treeview.get_dest_row_at_pos(x, y)
-            before = None
-            after = None
-            if drop_info:
-                path, position = drop_info
-                if position in types_before:
-                    before = items[path][3]
-                else:
-                    after = items[path][3]
-            else:
-                path = (len(items) - 1,)
-                after = items[path][3]
-            file_path = urllib.unquote(selection.data).strip()
-            if not file_path.startswith('file:'):
-                return
-            myfile = Gio.File(uri=file_path)
-            file_info = myfile.query_info(Gio.FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE)
-            content_type = file_info.get_content_type()
-            if content_type == 'application/x-desktop':
-                input_stream = myfile.read()
-                keyfile = GLib.KeyFile()
-                keyfile.load_from_data(input_stream.read())
-                self.editor.createItem(parent, before, after, KeyFile=keyfile)
-            elif content_type in ('application/x-shellscript', 'application/x-executable'):
-                self.editor.createItem(parent, before, after,
-                                       Name=os.path.split(file_path)[1].strip(),
-                                       Exec=file_path.replace('file://', '').strip(),
-                                       Terminal=False)
-        self.drag_data = None
-
     def on_item_tree_key_press_event(self, item_tree, event):
         if event.keyval == Gdk.KEY_Delete:
             self.on_edit_delete_activate(item_tree)
@@ -599,12 +464,6 @@ class MainWindow(object):
         elif isinstance(item, GMenu.TreeSeparator):
             self.editor.moveSeparator(item, item.get_parent(), after=after)
 
-    def on_mainwindow_undo(self, accelgroup, window, keyval, modifier):
-        self.editor.undo()
-
-    def on_mainwindow_redo(self, accelgroup, window, keyval, modifier):
-        self.editor.redo()
-
     def on_help_button_clicked(self, *args):
         Gtk.show_uri(Gdk.Screen.get_default(), "ghelp:user-guide#menu-editor", Gtk.get_current_event_time())
 
@@ -628,5 +487,4 @@ class MainWindow(object):
         self.loadUpdates()
 
     def quit(self):
-        self.editor.quit()
         Gtk.main_quit()
